@@ -1,6 +1,6 @@
 /*
  * mpstat: per-processor statistics
- * (C) 2000-2002 by Sebastien GODARD <sebastien.godard@wanadoo.fr>
+ * (C) 2000-2003 by Sebastien GODARD <sebastien.godard@wanadoo.fr>
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -45,9 +45,11 @@
 
 
 struct mp_stats *st_mp_cpu[DIM];
-unsigned int *cpu_bitmap;	/* Bit 0: Global; Bit 1: 1st proc; etc. */
+/* NOTE: Use array of _char_ for bitmaps to avoid endianness problems...*/
+unsigned char *cpu_bitmap;	/* Bit 0: Global; Bit 1: 1st proc; etc. */
 struct mp_timestamp st_mp_tstamp[DIM];
-int proc_used = -1;  /* Nb of processors on the machine. A value of 1 means two processors */
+/* Nb of processors on the machine. A value of 1 means two processors */
+int proc_used = -1;
 long interval = 0, count = 0;
 unsigned int flags = 0;
 struct tm loc_time;
@@ -95,10 +97,12 @@ void salloc_mp_cpu(int nr_cpus)
       memset(st_mp_cpu[i], 0, MP_STATS_SIZE * nr_cpus);
    }
 
-   if ((cpu_bitmap = (int *) malloc((nr_cpus >> 5) + 1)) == NULL) {
+   if ((cpu_bitmap = (unsigned char *) malloc((nr_cpus >> 3) + 1)) == NULL) {
       perror("malloc");
       exit(4);
    }
+
+   memset(cpu_bitmap, 0, (nr_cpus >> 3) + 1);
 }
 
 
@@ -119,41 +123,61 @@ void write_stats_avg(short curr, short dis)
 
    /* Print stats */
    if (dis)
-      printf(_("\nAverage:     CPU   %%user   %%nice %%system   %%idle    intr/s\n"));
+      printf(_("\nAverage:     CPU   %%user   %%nice %%system %%iowait   %%idle    intr/s\n"));
 
-   for (cpu = 0; cpu <= proc_used + (proc_used > 0); cpu++) {
+   /* Check if we want global average stats among all proc */
+   if (*cpu_bitmap & 1) {
+
+      printf(_("Average:     all"));
+
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
+	     SP_VALUE(st_mp_cpu[2]->cpu_user,   st_mp_cpu[curr]->cpu_user,   itv),
+	     SP_VALUE(st_mp_cpu[2]->cpu_nice,   st_mp_cpu[curr]->cpu_nice,   itv),
+	     SP_VALUE(st_mp_cpu[2]->cpu_system, st_mp_cpu[curr]->cpu_system, itv),
+	     SP_VALUE(st_mp_cpu[2]->cpu_iowait, st_mp_cpu[curr]->cpu_iowait, itv));
+
+      if (st_mp_cpu[curr]->cpu_idle < st_mp_cpu[2]->cpu_idle)
+	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
+      else
+	 printf("  %6.2f",
+		SP_VALUE(st_mp_cpu[2]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
+   }
+
+   /* Here, we reduce the interval value to one processor! */
+   itv /= (proc_used + 1);
+   if (!itv)
+      itv = 1;
+
+   if (*cpu_bitmap & 1) {
+      printf(" %9.2f\n",
+	  S_VALUE(st_mp_cpu[2]->irq, st_mp_cpu[curr]->irq, itv));
+   }
+
+   for (cpu = 1; cpu <= proc_used + 1; cpu++) {
 
       /* Check if we want stats about this proc */
-      if (!(*(cpu_bitmap + (cpu >> 5)) & (1 << (cpu & 0x1f))))
+      if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))))
 	 continue;
 
-      if (!cpu)
-	 printf(_("Average:     all"));
-      else
-	 printf(_("Average:    %4d"), cpu - 1);
+      printf(_("Average:    %4d"), cpu - 1);
 
       st_mp_cpu_i = st_mp_cpu[curr]  + cpu;
       st_mp_cpu_j = st_mp_cpu[2] + cpu;
 
-      printf("  %6.2f  %6.2f  %6.2f",
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
 	     SP_VALUE(st_mp_cpu_j->cpu_user,   st_mp_cpu_i->cpu_user,   itv),
 	     SP_VALUE(st_mp_cpu_j->cpu_nice,   st_mp_cpu_i->cpu_nice,   itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_system, st_mp_cpu_i->cpu_system, itv));
+	     SP_VALUE(st_mp_cpu_j->cpu_system, st_mp_cpu_i->cpu_system, itv),
+	     SP_VALUE(st_mp_cpu_j->cpu_iowait, st_mp_cpu_i->cpu_iowait, itv));
 
-      if (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle)	/* Handle buggy RTC (or kernels?) */
-	 printf("    %.2f", 0.0);
+      if (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle)
+	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
       else
 	 printf("  %6.2f",
 		SP_VALUE(st_mp_cpu_j->cpu_idle, st_mp_cpu_i->cpu_idle, itv));
 
-      if (!cpu) {
-	 itv /= (proc_used + 1);
-	 if (!itv)
-	    itv = 1;
-      }
-
       printf(" %9.2f\n",
-	     S_VALUE(st_mp_cpu[2]->irq, st_mp_cpu[curr]->irq, itv));
+	     S_VALUE(st_mp_cpu_j->irq, st_mp_cpu_i->irq, itv));
    }
 }
 
@@ -194,41 +218,62 @@ void write_stats(short curr, short dis)
 
    /* Print stats */
    if (dis)
-      printf(_("\n%-11s  CPU   %%user   %%nice %%system   %%idle    intr/s\n"), cur_time[!curr]);
+      printf(_("\n%-11s  CPU   %%user   %%nice %%system %%iowait   %%idle    intr/s\n"),
+	     cur_time[!curr]);
 
-   for (cpu = 0; cpu <= proc_used + (proc_used > 0); cpu++) {
+   /* Check if we want global stats among all proc */
+   if (*cpu_bitmap & 1) {
+
+      printf(_("%-11s  all"), cur_time[curr]);
+
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
+	     SP_VALUE(st_mp_cpu[!curr]->cpu_user,   st_mp_cpu[curr]->cpu_user,   itv),
+	     SP_VALUE(st_mp_cpu[!curr]->cpu_nice,   st_mp_cpu[curr]->cpu_nice,   itv),
+	     SP_VALUE(st_mp_cpu[!curr]->cpu_system, st_mp_cpu[curr]->cpu_system, itv),
+	     SP_VALUE(st_mp_cpu[!curr]->cpu_iowait, st_mp_cpu[curr]->cpu_iowait, itv));
+
+      if (st_mp_cpu[curr]->cpu_idle < st_mp_cpu[!curr]->cpu_idle)
+	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
+      else
+	 printf("  %6.2f",
+		SP_VALUE(st_mp_cpu[!curr]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
+   }
+
+   /* Here, we reduce the interval value to one processor! */
+   itv /= (proc_used + 1);
+   if (!itv)
+      itv = 1;
+
+   if (*cpu_bitmap & 1) {
+      printf(" %9.2f\n",
+	  S_VALUE(st_mp_cpu[!curr]->irq, st_mp_cpu[curr]->irq, itv));
+   }
+
+   for (cpu = 1; cpu <= proc_used + 1; cpu++) {
 
       /* Check if we want stats about this proc */
-      if (!(*(cpu_bitmap + (cpu >> 5)) & (1 << (cpu & 0x1f))))
+      if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))))
 	 continue;
 
-      if (!cpu)
-	 printf(_("%-11s  all"), cur_time[curr]);
-      else
-	 printf("%-11s %4d", cur_time[curr], cpu - 1);
+      printf("%-11s %4d", cur_time[curr], cpu - 1);
 
       st_mp_cpu_i = st_mp_cpu[curr]  + cpu;
       st_mp_cpu_j = st_mp_cpu[!curr] + cpu;
 
-      printf("  %6.2f  %6.2f  %6.2f",
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
 	     SP_VALUE(st_mp_cpu_j->cpu_user,   st_mp_cpu_i->cpu_user,   itv),
 	     SP_VALUE(st_mp_cpu_j->cpu_nice,   st_mp_cpu_i->cpu_nice,   itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_system, st_mp_cpu_i->cpu_system, itv));
+	     SP_VALUE(st_mp_cpu_j->cpu_system, st_mp_cpu_i->cpu_system, itv),
+	     SP_VALUE(st_mp_cpu_j->cpu_iowait, st_mp_cpu_i->cpu_iowait, itv));
 
-      if (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle)	/* Handle buggy RTC (or kernels?) */
-	 printf("    %.2f", 0.0);
+      if (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle)
+	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
       else
 	 printf("  %6.2f",
 		SP_VALUE(st_mp_cpu_j->cpu_idle, st_mp_cpu_i->cpu_idle, itv));
 
-      if (!cpu) {
-	 itv /= (proc_used + 1);
-	 if (!itv)
-	    itv = 1;
-      }
-
       printf(" %9.2f\n",
-	     S_VALUE(st_mp_cpu[!curr]->irq, st_mp_cpu[curr]->irq, itv));
+	  S_VALUE(st_mp_cpu_j->irq, st_mp_cpu_i->irq, itv));
    }
 }
 
@@ -243,7 +288,7 @@ void read_proc_stat(short curr)
    struct mp_stats *st_mp_cpu_i;
    static char line[80];
    unsigned int cc_user, cc_nice, cc_system;
-   unsigned long cc_idle;
+   unsigned long cc_idle, cc_iowait;
    int proc_nb;
 
    /* Open stat file */
@@ -256,34 +301,50 @@ void read_proc_stat(short curr)
 
       if (!strncmp(line, "cpu ", 4)) {
 	 /*
-	  * Read the number of jiffies spent in user, nice, system and idle mode among all proc.
-	  * CPU usage is not reduced to one processor to avoid rounding problems.
+	  * Read the number of jiffies spent in user, nice, system, idle
+	  * and iowait mode among all proc. CPU usage is not reduced to one
+	  * processor to avoid rounding problems.
 	  */
-	 sscanf(line + 5, "%u %u %u %lu",
+	 st_mp_cpu[curr]->cpu_iowait = 0;	/* For non 2.5 machines */
+	 sscanf(line + 5, "%u %u %u %lu %lu",
 		&(st_mp_cpu[curr]->cpu_user),   &(st_mp_cpu[curr]->cpu_nice),
-		&(st_mp_cpu[curr]->cpu_system), &(st_mp_cpu[curr]->cpu_idle));
+		&(st_mp_cpu[curr]->cpu_system), &(st_mp_cpu[curr]->cpu_idle),
+		&(st_mp_cpu[curr]->cpu_iowait));
 
 	 /*
-	  * Compute the uptime of the system in jiffies (1/100ths of a second if HZ=100).
+	  * Compute the uptime of the system in jiffies (1/100ths of a second
+	  * if HZ=100).
 	  * Machine uptime is multiplied by the number of processors here.
 	  */
-	 st_mp_tstamp[curr].uptime = st_mp_cpu[curr]->cpu_user   + st_mp_cpu[curr]->cpu_nice +
-	                             st_mp_cpu[curr]->cpu_system + st_mp_cpu[curr]->cpu_idle;
+	 st_mp_tstamp[curr].uptime = st_mp_cpu[curr]->cpu_user +
+	                             st_mp_cpu[curr]->cpu_nice +
+	                             st_mp_cpu[curr]->cpu_system +
+	                             st_mp_cpu[curr]->cpu_idle +
+	                             st_mp_cpu[curr]->cpu_iowait;
       }
 
       else if (!strncmp(line, "cpu", 3)) {
-	 if (proc_used > 0) {
-	    /*
-	     * Read the number of jiffies spent in user, nice, system and idle mode for current proc.
-	     * This is done only on SMP machines.
-	     */
-	    sscanf(line + 3, "%d %u %u %u %lu", &proc_nb, &cc_user, &cc_nice, &cc_system, &cc_idle);
+	 /*
+	  * Read the number of jiffies spent in user, nice, system, idle
+	  * and iowait mode for current proc.
+	  * This is done only on SMP machines.
+	  */
+	 cc_iowait = 0;	/* For non 2.5 machines */
+	 sscanf(line + 3, "%d %u %u %u %lu %lu",
+		&proc_nb,
+		&cc_user, &cc_nice, &cc_system, &cc_idle, &cc_iowait);
+	 if (proc_nb <= proc_used) {
 	    st_mp_cpu_i = st_mp_cpu[curr] + proc_nb + 1;
 	    st_mp_cpu_i->cpu_user   = cc_user;
 	    st_mp_cpu_i->cpu_nice   = cc_nice;
 	    st_mp_cpu_i->cpu_system = cc_system;
 	    st_mp_cpu_i->cpu_idle   = cc_idle;
+	    st_mp_cpu_i->cpu_iowait = cc_iowait;
 	 }
+	 /* else:
+	  * Additional CPUs have been dynamically registered in /proc/stat.
+	  * mpstat won't crash, but the CPU stats might be false...
+	  */
       }
 
       else if (!strncmp(line, "intr ", 5))
@@ -350,11 +411,12 @@ int main(int argc, char **argv)
 
    /* How many processors on this machine ? */
    get_nb_proc_used(&proc_used, ~0);
+
    /*
     * proc_used: a value of 1 means there are 2 processors (0 and 1).
     * In this case, we have to allocate 3 structures: global, proc0 and proc1.
     */
-   salloc_mp_cpu(proc_used + 1 + (proc_used > 0));
+   salloc_mp_cpu(proc_used + 2);
 
    while (++opt < argc) {
 
@@ -362,17 +424,18 @@ int main(int argc, char **argv)
 	 usage(argv[0]);
 
       else if (!strcmp(argv[opt], "-P")) {
-	 if (!proc_used) {
-	    fprintf(stderr, _("Not an SMP machine...\n"));
-	    exit(1);
-	 }
+	 /* '-P ALL' can be used on UP machines */
 	 if (argv[++opt]) {
 	    opt_used = 1;
 	    dis_hdr++;
-	    if (!strcmp(argv[opt], K_ALL) || !strcmp(argv[opt], "-1")) {
-	       dis_hdr = 9;
-	       /* Set bit for every processor */
-	       memset(cpu_bitmap, ~0, ((proc_used + 1 + (proc_used > 0)) >> 5) + 1);
+	    if (!strcmp(argv[opt], K_ALL)) {
+	       if (proc_used)
+		  dis_hdr = 9;
+	       /*
+		* Set bit for every processor.
+		* Also indicate to display stats for CPU 'all'.
+		*/
+	       memset(cpu_bitmap, 0xff, ((proc_used + 1 + (proc_used > 0)) >> 3) + 1);
 	    }
 	    else {
 	       if (strspn(argv[opt], DIGITS) != strlen(argv[opt]))
@@ -383,7 +446,7 @@ int main(int argc, char **argv)
 		  exit(1);
 	       }
 	       i++;
-	       *(cpu_bitmap + (i >> 5)) |= 1 << (i & 0x1f);
+	       *(cpu_bitmap + (i >> 3)) |= 1 << (i & 0x07);
 	    }
 	 }
 	 else
@@ -438,8 +501,7 @@ int main(int argc, char **argv)
 
    /* Read stats */
    read_proc_stat(0);
-   if (proc_used > 0)
-      read_interrupts_stat(0);
+   read_interrupts_stat(0);
 
    if (!interval || WANT_BOOT_STATS(flags)) {
       /* Display since boot time */
@@ -449,7 +511,7 @@ int main(int argc, char **argv)
 
       st_mp_tstamp[1].uptime = 0;
 
-      memset(st_mp_cpu[1], 0, MP_STATS_SIZE * (proc_used + 1 + (proc_used > 0)));
+      memset(st_mp_cpu[1], 0, MP_STATS_SIZE * (proc_used + 2));
 
       write_stats(0, DISP_HDR);
       exit(0);
@@ -465,14 +527,14 @@ int main(int argc, char **argv)
 
    st_mp_tstamp[2].uptime = st_mp_tstamp[0].uptime;
 
-   memcpy(st_mp_cpu[2], st_mp_cpu[0], MP_STATS_SIZE * (proc_used + 1 + (proc_used > 0)));
+   memcpy(st_mp_cpu[2], st_mp_cpu[0], MP_STATS_SIZE * (proc_used + 2));
 
    pause();
 
    /* Main loop */
    do {
 
-      /* Resetting the structure is not needed since all the fields will be filled */
+      /* Resetting the structure not needed since every fields will be set */
 
       /* Save time */
       get_localtime(&loc_time);
@@ -483,8 +545,7 @@ int main(int argc, char **argv)
 
       /* Read stats */
       read_proc_stat(curr);
-      if (proc_used > 0)
-	 read_interrupts_stat(curr);
+      read_interrupts_stat(curr);
 
       /* Write stats */
       if (!dis_hdr) {
